@@ -309,7 +309,7 @@ class Stream(object):
 
         If no input or output device (or True) is specified, the
         default input/output device is taken. For input/output-only
-        streams, proved None or False as input/output-device.
+        streams, provide None or False as input/output-device.
 
         The output/output device is merely a dictionary of parameters.
         Customize those parameters for more precise control over the
@@ -348,27 +348,39 @@ class Stream(object):
         if output_device is True:
             output_device = default_output_device()
 
-        stream_parameters_in = ffi.new("PaStreamParameters*",
-                                       ( input_device['device_index'],
-                                         input_device['input_channels'],
-                                         _np2pa[input_device['sample_format']],
-                                         input_device['input_latency'],
-                                         ffi.NULL ))
-        self.input_format = input_device['sample_format']
-        self.input_channels = stream_parameters_in.channelCount
-        if stream_parameters_in and not input_device['interleaved_data']:
-            stream_parameters_in.sampleFormat |= 0x80000000
+        if input_device:
+            stream_parameters_in = \
+                ffi.new("PaStreamParameters*",
+                        ( input_device['device_index'],
+                          input_device['input_channels'],
+                          _np2pa[input_device['sample_format']],
+                          input_device['input_latency'],
+                          ffi.NULL ))
+            self.input_format = input_device['sample_format']
+            self.input_channels = stream_parameters_in.channelCount
+            if stream_parameters_in and not input_device['interleaved_data']:
+                stream_parameters_in.sampleFormat |= 0x80000000
+        else:
+            stream_parameters_in = ffi.NULL
+            self.input_format = None
+            self.input_channels = 0
 
-        stream_parameters_out = ffi.new("PaStreamParameters*",
-                                        ( output_device['device_index'],
-                                          output_device['output_channels'],
-                                          _np2pa[output_device['sample_format']],
-                                          output_device['output_latency'],
-                                          ffi.NULL ))
-        self.output_format = output_device['sample_format']
-        self.output_channels = stream_parameters_out.channelCount
-        if stream_parameters_out and not output_device['interleaved_data']:
-            stream_parameters_out.sampleFormat |= 0x80000000
+        if output_device:
+            stream_parameters_out = \
+                ffi.new("PaStreamParameters*",
+                        ( output_device['device_index'],
+                          output_device['output_channels'],
+                          _np2pa[output_device['sample_format']],
+                          output_device['output_latency'],
+                          ffi.NULL ))
+            self.output_format = output_device['sample_format']
+            self.output_channels = stream_parameters_out.channelCount
+            if stream_parameters_out and not output_device['interleaved_data']:
+                stream_parameters_out.sampleFormat |= 0x80000000
+        else:
+            stream_parameters_out = ffi.NULL
+            self.output_format = None
+            self.output_channels = 0
 
         stream_flags = 0
         if 'no_clipping' in flags:
@@ -383,13 +395,19 @@ class Stream(object):
         if callback:
             def callback_stub(input_ptr, output_ptr, num_frames, time_struct,
                       status_flags, user_data):
-                num_bytes = (self.input_channels * _npsizeof[self.input_format]
-                             * num_frames)
-                input_data = np.fromstring(ffi.buffer(input_ptr, num_bytes),
-                                           dtype=self.input_format,
-                                           count=num_frames*self.input_channels)
-                input_data = np.reshape(input_data,
-                                        (num_frames, self.input_channels))
+                if self.input_channels > 0:
+                    num_bytes = (self.input_channels *
+                                 _npsizeof[self.input_format] *
+                                 num_frames)
+                    input_data = \
+                        np.fromstring(ffi.buffer(input_ptr, num_bytes),
+                                      dtype=self.input_format,
+                                      count=num_frames*self.input_channels)
+                    input_data = np.reshape(input_data,
+                                            (num_frames, self.input_channels))
+                else:
+                    input_data = None
+
                 time_info = {'input_adc_time': time_struct.inputBufferAdcTime,
                              'current_time': time_struct.currentTime,
                              'output_dac_time': time_struct.outputBufferDacTime}
@@ -397,11 +415,15 @@ class Stream(object):
                 output_data, flag = callback(input_data, num_frames,
                                              time_info, status_flags)
 
-                num_frames or len(output_data)
-                if output_data.dtype != self.output_format:
-                    output_data = np.array(output_data, dtype=self.output_format)
-                output_buffer = ffi.buffer(output_ptr, num_bytes)
-                output_buffer[:] = output_data.flatten().tostring()
+                if self.output_channels > 0:
+                    num_bytes = (self.output_channels *
+                                 _npsizeof[self.output_format] *
+                                 num_frames)
+                    if output_data.dtype != self.output_format:
+                        output_data = np.array(output_data,
+                                               dtype=self.output_format)
+                    output_buffer = ffi.buffer(output_ptr, num_bytes)
+                    output_buffer[:] = output_data.flatten().tostring()
                 return flag
 
             self._callback = \
@@ -418,6 +440,13 @@ class Stream(object):
                                 ffi.NULL)
         self._handle_error(err)
 
+        # set some stream information
+        self.sample_rate = sample_rate
+        self.block_length = block_length
+        info = _pa.Pa_GetStreamInfo(self._stream[0])
+        self.input_latency = info.inputLatency,
+        self.output_latency = info.outputLatency,
+
         if finished_callback:
             def finished_callback_stub(userData):
                 finished_callback()
@@ -444,17 +473,17 @@ class Stream(object):
         # At program shutdown, _pa is sometimes deleted before this
         # function is called. However, in that case, Pa_Terminate
         # already took care of closing all dangling streams.
-        if _pa:
+        if _pa and self._stream:
             self._handle_error(_pa.Pa_CloseStream(self._stream[0]))
+            self._stream = None
 
     def __enter__(self):
-        # A context manager for convenience
         self.start()
         return self
 
     def __exit__(self, type, value, tb):
-        # A context manager for convenience
         self.stop()
+        self.__del__()
 
     def start(self):
         """Commence audio processing.
@@ -512,33 +541,6 @@ class Stream(object):
         """The number of frames that can be read without waiting."""
         return _pa.Pa_GetStreamWriteAvailable(self._stream[0])
 
-    def info(self):
-        """Retrieve unchanging information about the open stream.
-
-        Returns a dictionary containing four fields:
-
-        struct_version: Always 1
-
-        input_latency: The most accurate estimate of the input latency
-        of the stream in seconds. The value will be zero for
-        output-only streams.
-
-        output_latency: The most accurate estimate of the output
-        latency of the stream in seconds. The value will be zero for
-        input-only streams.
-
-        sample_rate: The sample rate of the stream in Hz. In cases
-        where the hardware sample rate is inaccurate and portaudio is
-        aware of it, the value of this field may be different from the
-        sample_rate parameter passed to the stream constructor.
-
-        """
-        info = _pa.Pa_GetStreamInfo(self._stream[0])
-        return { 'struct_version': info.structVersion,
-                 'input_latency': info.inputLatency,
-                 'output_latency': info.outputLatency,
-                 'sample_rate': info.sampleRate }
-
     def time(self):
         """Returns the current stream time in seconds.
 
